@@ -44,7 +44,7 @@ class Simulate:
                     army_b: Optional[Army], 
                     config = None,
                     combat_system: Optional[CombatSystem] = None):
-        self.N = 5000
+        self.N = 2500
         self.army_a = army_a
         self.army_b = army_b
         self.battle_type = 'land'
@@ -109,10 +109,10 @@ class Simulate:
             raise RuntimeError("No Config for combat system found")
 
 
-        for n in tqdm(range(self.N)):
+        for n in (range(self.N)):
             self.cur_n = n
-            battle = Battle(copy.copy(self.army_a), 
-                            copy.copy(self.army_b),
+            battle = Battle(copy.deepcopy(self.army_a), 
+                            copy.deepcopy(self.army_b),
                             self.config)
             status = battle.run(combat_system=self.combat_system)
             
@@ -130,11 +130,20 @@ class Simulate:
                 app.spinner.stop()
 
                 self.eval_statistics()
-                app.results.text = self.get_report()
+                #app.results.text = self.get_report_short()
+                app.win_loss_dist = self.intermediate_statistics()
+                app.stats_a_ground = self.metrics['stats_a_ground']
+                app.stats_b_ground = self.metrics['stats_b_ground']
+                app.stats_a_air = self.metrics['stats_a_air']
+                app.stats_b_air = self.metrics['stats_b_air']
+                app.draw_chart()
                 return
 
-            if n % 50 == 0 and n != self.N:
-                app.results.text = self.intermediate_statistics()
+            if n % 150 == 0 and n != self.N:
+                #app.results.text = self.intermediate_statistics()
+                app.win_loss_dist = self.intermediate_statistics()
+                app.draw_chart()
+
                 yield 0.01
 
 
@@ -155,7 +164,7 @@ class Simulate:
         else:
             return None
 
-    def running_stats(self, status, eps=0.5):
+    def running_stats(self, status, eps=0.5, min_run=500):
         self.stats[status] += 1
         N = (self.cur_n + 1)
         x = self.stats / N
@@ -173,7 +182,7 @@ class Simulate:
         self.M2 += delta * delta2
         (mean, variance, sampleVariance) = (self.moving_mean, self.M2 / N, self.M2 / (N - 1))
 
-        if np.max(sampleVariance)**(1/2) < 0.01 and self.cur_n > 250:
+        if np.max(sampleVariance)**(1/2) < 0.01 and self.cur_n > min_run:
             self.moving_stats_last = self.moving_mean[:]
             return True
         else:
@@ -182,53 +191,6 @@ class Simulate:
 
 
 
-    def run_async(self, combat_system: CombatSystem, config=None, armyA: Optional[Army]=None, armyB: Optional[Army]=None) -> int:
-        if armyA is not None:
-            self.army_a = armyA
-        if armyB is not None:
-            self.army_b = armyB
-
-        self.update_battle_type()
-
-        troops_a, troops_b = 0, 0
-        for T in ['sea', 'land', 'air']:
-            troops_a += self.army_a.units[T].sum() 
-            troops_b += self.army_b.units[T].sum() 
-
-        if troops_a == 0 or troops_b == 0:
-            yield (False,100)
-
-        if combat_system is CombatSystem.WarRoomV2 and config is None:
-            logging.warning("NO CONFIG provided falling back to default config!!!!")
-            config = wr20_vaniilla_options
-        elif config is None:
-            raise RuntimeError("No Config for combat system found")
-
-        for n in tqdm(range(self.N)):
-            self.cur_n = n
-            battle = Battle(copy.copy(self.army_a), 
-                            copy.copy(self.army_b),
-                            config)
-            status = battle.run(combat_system=combat_system)
-            #print(status)
-            abrt = self.running_stats(status)
-            self.statistics.append(status)
-
-            for side in ['A', 'B']:
-                for type in ['land', 'air', 'sea']:
-                    self.survivors[side][type].append(battle.army[side].units[type])
-                    if type == 'sea' and battle.army[side].submerged > 0:
-                        self.survivors[side][type][-1][0] += battle.army[side].submerged
-            if abrt:
-                yield (False, 100)
-
-            #if n % 50 == 0 and q_intermediate is not None and n != self.N and q_intermediate.empty():
-                #q_intermediate.put(self.intermediate_statistics())
-
-            yield (True, int(self.N / n))
-
-
-
     def run(self, combat_system: CombatSystem, config=None, armyA: Optional[Army]=None, armyB: Optional[Army]=None) -> int:
         
         if armyA is not None:
@@ -261,29 +223,32 @@ class Simulate:
             #print(status)
             abrt = self.running_stats(status)
             self.statistics.append(status)
+            print(battle.army['B'].units['air'])
 
             for side in ['A', 'B']:
                 for type in ['land', 'air', 'sea']:
                     self.survivors[side][type].append(battle.army[side].units[type])
+
                     if type == 'sea' and battle.army[side].submerged > 0:
                         self.survivors[side][type][-1][0] += battle.army[side].submerged
             if abrt:
                 return
 
 
-
-
     def intermediate_statistics(self):
-
         N = (self.cur_n + 1)
         x = self.stats / N
         x[np.isnan(x)] = 0
+        return np.array([x[1],x[0],x[2],x[3]])
 
+
+    def intermediate_statistics_as_text(self):
+        x = self.intermediate_statistics()
         report = StringIO(newline=os.linesep)
 
         report.write(f"Results:{os.linesep}")
-        report.write(f"{'A Won:':<20}{x[1]:.2f}{os.linesep}")
-        report.write(f"{'B Won:':<20}{x[0]:.2f}{os.linesep}") 
+        report.write(f"{'A Won:':<20}{x[0]:.2f}{os.linesep}")
+        report.write(f"{'B Won:':<20}{x[1]:.2f}{os.linesep}") 
         report.write(f"{'Draw:':<20}{x[2]:.2f}{os.linesep}")
         report.write(f"{'Mutual Annihilation:':<20}{x[3]:.2f}{os.linesep}")
 
@@ -312,10 +277,10 @@ class Simulate:
         idx_games_draw  = np.argwhere(self.statistics==2)
         idx_games_won_none = np.argwhere(self.statistics==3)
 
-        avg_surv_a   = np.squeeze(np.mean(self.survivors_a[idx_games_won_a], 0))    if idx_games_won_a.any()    else np.array([0,0,0,0,0])
-        avg_surv_b   = np.squeeze(np.mean(self.survivors_b[idx_games_won_b], 0))    if idx_games_won_b.any()    else np.array([0,0,0,0,0])
-        avg_draw_a   = np.squeeze(np.mean(self.survivors_a[idx_games_draw], 0))     if idx_games_draw.any()     else np.array([0,0,0,0,0])
-        avg_draw_b   = np.squeeze(np.mean(self.survivors_b[idx_games_draw], 0))     if idx_games_draw.any()     else np.array([0,0,0,0,0])
+        avg_surv_a   = np.squeeze(np.mean(self.survivors_a[idx_games_won_a], 0))    if idx_games_won_a.any()    else np.zeros(10)
+        avg_surv_b   = np.squeeze(np.mean(self.survivors_b[idx_games_won_b], 0))    if idx_games_won_b.any()    else np.zeros(10)
+        avg_draw_a   = np.squeeze(np.mean(self.survivors_a[idx_games_draw], 0))     if idx_games_draw.any()     else np.zeros(10)
+        avg_draw_b   = np.squeeze(np.mean(self.survivors_b[idx_games_draw], 0))     if idx_games_draw.any()     else np.zeros(10)
 
         self.metrics['avg_surv_a'] = avg_surv_a
         self.metrics['avg_surv_b'] = avg_surv_b
@@ -355,6 +320,31 @@ class Simulate:
         top_varations_draw_a, distr_draw_a = top_variations(self.survivors_a[idx_games_draw])
         top_varations_draw_b, distr_draw_b = top_variations(self.survivors_b[idx_games_draw])
 
+
+        units_a = np.array(self.survivors['A'][self.battle_type])
+        units_b = np.array(self.survivors['B'][self.battle_type])
+        units_a_air = np.array(self.survivors['A']['air'])
+        units_b_air = np.array(self.survivors['B']['air'])
+
+        def unit_hist(arr):
+            stats = []
+            for i in range(arr.shape[1]):
+                num, cnt = np.unique(arr[:,i], return_counts=True)
+                cnt = cnt / arr.shape[0]
+                stats.append((num,cnt))
+            return stats
+
+        stats_a_ground = unit_hist(units_a)
+        stats_b_ground = unit_hist(units_b)
+        stats_a_air = unit_hist(units_a_air)
+        stats_b_air = unit_hist(units_b_air)
+
+        self.metrics['stats_a_ground'] = stats_a_ground
+        self.metrics['stats_b_ground'] = stats_b_ground
+        self.metrics['stats_a_air'] = stats_a_air
+        self.metrics['stats_b_air'] = stats_b_air
+
+
         self.metrics['outcomes_won_a']  = {'variations': top_varations_won_a, 'distribution': distr_won_a}
         self.metrics['outcomes_won_b']  = {'variations': top_varations_won_b, 'distribution': distr_won_b}
         self.metrics['outcomes_draw_a'] = {'variations': top_varations_draw_a, 'distribution': distr_draw_a}
@@ -499,6 +489,7 @@ class Simulate:
             str += f"{'AIR Red':<10}"
             buf.write(str+os.linesep)
             return buf.getvalue()
+
         N = self.cur_n+1
         report.write("--------------------------------------------------------------"+os.linesep)
         report.write(f"Results:{os.linesep}")
